import * as React from 'react';
import { ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, isValidProp, toSpinal, getModifiedProps } from "igniteui-react-core";
import { FontInfo } from "igniteui-react-core";
import { ReactRenderer } from "igniteui-react-core";
import { delegateCombine, delegateRemove } from "igniteui-react-core";
import { DatePicker } from './DatePicker';
import { IgrValueChangedEventArgs } from "./igr-value-changed-event-args";
import { IgrGotFocusEventArgs } from "./igr-got-focus-event-args";
import { IgrLostFocusEventArgs } from "./igr-lost-focus-event-args";
import { IgrInputChangeEventArgs } from "./igr-input-change-event-args";
import { IgrKeyEventArgs } from "igniteui-react-core";
import { ControlDisplayDensity_$type } from "igniteui-react-core";
import { BaseControlTheme_$type } from "igniteui-react-core";
import { DateFormats_$type } from "./DateFormats";
import { DayOfWeek_$type } from "./DayOfWeek";
import { FirstWeek_$type } from "./FirstWeek";
export class IgrDatePicker extends React.Component {
    constructor(props) {
        super(props);
        this._implementation = null;
        this.__p = null;
        this._hasUserValues = new Set();
        this._stylingContainer = null;
        this._stylingParent = null;
        this._inStyling = false;
        this._valueChanged = null;
        this._valueChanged_wrapped = null;
        this._gotFocus = null;
        this._gotFocus_wrapped = null;
        this._lostFocus = null;
        this._lostFocus_wrapped = null;
        this._keyDown = null;
        this._keyDown_wrapped = null;
        this._changing = null;
        this._changing_wrapped = null;
        if (this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
        }
        this._getMainRef = this._getMainRef.bind(this);
        this._implementation = this.createImplementation();
        var datePicker = this.i;
        this._datePicker = datePicker;
        datePicker.notifySizeChanged();
        if (props) {
            this.initializeProperties();
        }
    }
    set height(value) {
        this._height = value;
        if (this._elRef) {
            this._elRef.style.height = value;
            this._datePicker.notifySizeChanged();
        }
    }
    get height() {
        return this._height;
    }
    set width(value) {
        this._width = value;
        if (this._elRef) {
            this._elRef.style.height = value;
            this._datePicker.notifySizeChanged();
        }
    }
    get width() {
        return this._width;
    }
    _getMainRef(ref) {
        this._elRef = ref;
        if (this._elRef != null) {
            this._renderer = new ReactRenderer(this._elRef, document, false, null);
            this._datePicker.provideContainer(this._renderer);
        }
    }
    render() {
        let children = [];
        let inputContainer = React.createElement("div", {
            key: "inputContainer"
        });
        children.push(inputContainer);
        let popupContainer = React.createElement("div", {
            key: "popupContainer"
        });
        children.push(popupContainer);
        let div = React.createElement("div", {
            className: "ig-date-picker igr-date-picker",
            ref: this._getMainRef,
            children: children
        });
        return div;
    }
    shouldComponentUpdate(nextProps, nextState) {
        const mod = getModifiedProps(this.props, nextProps);
        for (const p of Object.keys(mod)) {
            if (isValidProp(this, p)) {
                this[p] = mod[p];
            }
        }
        return true;
    }
    initializeProperties() {
        for (const p of Object.keys(this.props)) {
            if (isValidProp(this, p)) {
                this[p] = this.props[p];
            }
        }
    }
    // supports angular themes or custom properties set in CSS
    updateStyle() {
        this._styling(this._elRef, this);
    }
    componentWillUnmount() {
        this._datePicker.destroy();
        if (this._renderer) {
            this._renderer.destroy();
        }
    }
    componentDidMount() {
        this._elRef.style.width = this._width ? this._width : "";
        this._elRef.style.height = this._height ? this._height : "";
        this._datePicker.notifySizeChanged();
        this.initializeContent();
    }
    initializeContent() {
        this.updateStyle();
        this.i.notifySizeChanged();
    }
    createImplementation() {
        return new DatePicker();
    }
    get i() {
        return this._implementation;
    }
    /**
 * Gets or Sets the property name that contains the values.
*/
    get value() {
        return this.i.a0;
    }
    set value(v) {
        this.i.a0 = v;
    }
    /**
     * Gets or Sets the property name that contains the values.
    */
    get today() {
        return this.i.az;
    }
    set today(v) {
        this.i.az = v;
    }
    /**
     * Gets or Sets the property name that contains the label.
    */
    get label() {
        return this.i.bp;
    }
    set label(v) {
        this.i.bp = v;
    }
    /**
     * Gets or sets the color to use for the text.
    */
    get labelTextColor() {
        return brushToString(this.i.c9);
    }
    set labelTextColor(v) {
        this.i.c9 = stringToBrush(v);
    }
    /**
     * Gets or sets the font to use for the combobox.
    */
    get labelTextStyle() {
        if (this.i.ac == null) {
            return null;
        }
        return this.i.ac.fontString;
    }
    set labelTextStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.ac = fi;
    }
    /**
     * Gets or Sets the property name that contains the placeholder.
    */
    get placeholder() {
        return this.i.bw;
    }
    set placeholder(v) {
        this.i.bw = v;
    }
    /**
     * Gets or Sets the property name that contains the MinDate.
    */
    get minDate() {
        return this.i.ay;
    }
    set minDate(v) {
        this.i.ay = v;
    }
    /**
     * Gets or Sets the property name that contains the MaxDate.
    */
    get maxDate() {
        return this.i.ax;
    }
    set maxDate(v) {
        this.i.ax = v;
    }
    /**
     * Gets or sets the display density to use for the date pcicker.
    */
    get density() {
        return this.i.d;
    }
    set density(v) {
        this.i.d = ensureEnum(ControlDisplayDensity_$type, v);
    }
    /**
     * Gets or sets the base built in theme to use for the date picker.
    */
    get baseTheme() {
        return this.i.b;
    }
    set baseTheme(v) {
        this.i.b = ensureEnum(BaseControlTheme_$type, v);
    }
    /**
     * Gets or sets the font to use for the combobox.
    */
    get textStyle() {
        if (this.i.ad == null) {
            return null;
        }
        return this.i.ad.fontString;
    }
    set textStyle(v) {
        let fi = new FontInfo();
        fi.fontString = v;
        this.i.ad = fi;
    }
    /**
     * Gets or Sets the text color
    */
    get textColor() {
        return brushToString(this.i.da);
    }
    set textColor(v) {
        this.i.da = stringToBrush(v);
    }
    /**
     * Gets or Sets the text color
    */
    get iconColor() {
        return brushToString(this.i.c8);
    }
    set iconColor(v) {
        this.i.c8 = stringToBrush(v);
    }
    /**
     * Gets or sets the ShowClearButton property to detirmine if the clear button is shown
    */
    get showClearButton() {
        return this.i.aq;
    }
    set showClearButton(v) {
        this.i.aq = ensureBool(v);
    }
    /**
     * Gets or sets the ShowTodayButton property to detirmine if the today button is shown
    */
    get showTodayButton() {
        return this.i.ar;
    }
    set showTodayButton(v) {
        this.i.ar = ensureBool(v);
    }
    /**
     * Gets or sets the AllowTextInput property to detirmine if entering text into the input is allowed
    */
    get allowTextInput() {
        return this.i.ao;
    }
    set allowTextInput(v) {
        this.i.ao = ensureBool(v);
    }
    /**
     * Gets or sets the AllowTextInput property to detirmine if entering text into the input is allowed
    */
    get openOnFocus() {
        return this.i.ap;
    }
    set openOnFocus(v) {
        this.i.ap = ensureBool(v);
    }
    /**
     * Gets or sets the FirstDayOfWeek property to detirmine first day of the week
    */
    get firstDayOfWeek() {
        return this.i.n;
    }
    set firstDayOfWeek(v) {
        this.i.n = ensureEnum(DayOfWeek_$type, v);
    }
    /**
     * Gets or sets the FirstWeekOfYear property to detirmine first week of the year
    */
    get firstWeekOfYear() {
        return this.i.p;
    }
    set firstWeekOfYear(v) {
        this.i.p = ensureEnum(FirstWeek_$type, v);
    }
    /**
     * Gets or sets the ShowWeekNumbers property to detirmine if the week numbers are shown
    */
    get showWeekNumbers() {
        return this.i.as;
    }
    set showWeekNumbers(v) {
        this.i.as = ensureBool(v);
    }
    /**
     * Gets or sets the date time format to use for this column. If FormatString is specificied this value is ignored.
    */
    get dateFormat() {
        return this.i.i;
    }
    set dateFormat(v) {
        this.i.i = ensureEnum(DateFormats_$type, v);
    }
    /**
     * Gets or sets the Format property to detirmine the format of the date in the input
    */
    get formatString() {
        return this.i.bm;
    }
    set formatString(v) {
        this.i.bm = v;
    }
    get hasUserValues() {
        return this._hasUserValues;
    }
    __m(propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    }
    _styling(container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        let genericPrefix = "";
        let typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("DatePicker");
        let additionalPrefixes = [];
        let prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        let b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            let basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            let parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            let parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    }
    select() {
        this.i.cw();
    }
    /**
     * Exports visual information about the current state of the grid.
    
    */
    exportVisualModel() {
        let iv = this.i.a3();
        return (iv);
    }
    /**
     * Returns a serialized copy of the exported visual model
    
    */
    exportSerializedVisualModel() {
        let iv = this.i.bj();
        return (iv);
    }
    /**
     * Called when date is selected.
    */
    get valueChanged() {
        return this._valueChanged;
    }
    set valueChanged(ev) {
        if (this._valueChanged_wrapped !== null) {
            this.i.valueChanged = delegateRemove(this.i.valueChanged, this._valueChanged_wrapped);
            this._valueChanged_wrapped = null;
            this._valueChanged = null;
        }
        this._valueChanged = ev;
        this._valueChanged_wrapped = (o, e) => {
            let outerArgs = new IgrValueChangedEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeValueChanged) {
                this.beforeValueChanged(this, outerArgs);
            }
            if (this._valueChanged) {
                this._valueChanged(this, outerArgs);
            }
        };
        this.i.valueChanged = delegateCombine(this.i.valueChanged, this._valueChanged_wrapped);
    }
    get gotFocus() {
        return this._gotFocus;
    }
    set gotFocus(ev) {
        if (this._gotFocus_wrapped !== null) {
            this.i.gotFocus = delegateRemove(this.i.gotFocus, this._gotFocus_wrapped);
            this._gotFocus_wrapped = null;
            this._gotFocus = null;
        }
        this._gotFocus = ev;
        this._gotFocus_wrapped = (o, e) => {
            let outerArgs = new IgrGotFocusEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeGotFocus) {
                this.beforeGotFocus(this, outerArgs);
            }
            if (this._gotFocus) {
                this._gotFocus(this, outerArgs);
            }
        };
        this.i.gotFocus = delegateCombine(this.i.gotFocus, this._gotFocus_wrapped);
    }
    get lostFocus() {
        return this._lostFocus;
    }
    set lostFocus(ev) {
        if (this._lostFocus_wrapped !== null) {
            this.i.lostFocus = delegateRemove(this.i.lostFocus, this._lostFocus_wrapped);
            this._lostFocus_wrapped = null;
            this._lostFocus = null;
        }
        this._lostFocus = ev;
        this._lostFocus_wrapped = (o, e) => {
            let outerArgs = new IgrLostFocusEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeLostFocus) {
                this.beforeLostFocus(this, outerArgs);
            }
            if (this._lostFocus) {
                this._lostFocus(this, outerArgs);
            }
        };
        this.i.lostFocus = delegateCombine(this.i.lostFocus, this._lostFocus_wrapped);
    }
    get keyDown() {
        return this._keyDown;
    }
    set keyDown(ev) {
        if (this._keyDown_wrapped !== null) {
            this.i.keyDown = delegateRemove(this.i.keyDown, this._keyDown_wrapped);
            this._keyDown_wrapped = null;
            this._keyDown = null;
        }
        this._keyDown = ev;
        this._keyDown_wrapped = (o, e) => {
            let outerArgs = new IgrKeyEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeKeyDown) {
                this.beforeKeyDown(this, outerArgs);
            }
            if (this._keyDown) {
                this._keyDown(this, outerArgs);
            }
        };
        this.i.keyDown = delegateCombine(this.i.keyDown, this._keyDown_wrapped);
    }
    get changing() {
        return this._changing;
    }
    set changing(ev) {
        if (this._changing_wrapped !== null) {
            this.i.changing = delegateRemove(this.i.changing, this._changing_wrapped);
            this._changing_wrapped = null;
            this._changing = null;
        }
        this._changing = ev;
        this._changing_wrapped = (o, e) => {
            let outerArgs = new IgrInputChangeEventArgs();
            outerArgs._provideImplementation(e);
            if (this.beforeChanging) {
                this.beforeChanging(this, outerArgs);
            }
            if (this._changing) {
                this._changing(this, outerArgs);
            }
        };
        this.i.changing = delegateCombine(this.i.changing, this._changing_wrapped);
    }
}
//# sourceMappingURL=igr-date-picker.js.map
